from collections import defaultdict
from typing import Dict, Tuple

from ..apps.base import logger
from ..formats.base import BaseFile, read_block, must_open
from ..utils.range import Range


class AnchorFile(BaseFile):
    def __init__(self, filename, minsize=0):
        super().__init__(filename)
        self.blocks = list(self.iter_blocks(minsize=minsize))

    def iter_blocks(self, minsize=0):
        fp = open(self.filename)
        for _, lines in read_block(fp, "#"):
            lines = [x.split() for x in lines]
            if len(lines) >= minsize:
                yield lines

    def iter_pairs(self, minsize=0):
        block_id = -1
        for rows in self.iter_blocks(minsize=minsize):
            block_id += 1
            for row in rows:
                a, b = row[:2]
                yield a, b, block_id

    def make_ranges(self, order, clip=10):
        """Prepare anchors information into a set of ranges for chaining"""
        ranges = []
        block_pairs = defaultdict(dict)
        blocks = self.blocks
        for i, ib in enumerate(blocks):
            q, s, t = zip(*ib)
            if q[0] not in order:
                q, s = s, q

            r = make_range(q, s, t, i, order, block_pairs, clip=clip)
            ranges.append(r)

            assert q[0] in order
            if s[0] not in order:
                continue

            # is_self comparison
            q, s = s, q
            r = make_range(q, s, t, i, order, block_pairs, clip=clip)
            ranges.append(r)
        return ranges, block_pairs

    def filter_blocks(self, accepted: Dict[Tuple[str, str], str]):
        """
        Filter the blocks based on the accepted pairs. This is used to update
        the anchors so that they match the info in the LAST file.
        """
        new_blocks = []
        nremoved = 0
        ncorrected = 0
        nblocks_removed = 0
        for block in self.blocks:
            new_block = []
            for line in block:
                a, b, score = line
                pair = (a, b)
                if pair not in accepted:
                    nremoved += 1
                    continue
                av = accepted[pair]
                if score != av and score != av + "L":
                    score = av
                    ncorrected += 1
                new_block.append((a, b, score))
            if new_block:
                new_blocks.append(new_block)
            else:
                nblocks_removed += 1

        logger.debug("Removed %d existing anchors", nremoved)
        if nblocks_removed:
            logger.debug("Removed %d empty blocks", nblocks_removed)
        logger.debug("Corrected scores for %d anchors", ncorrected)
        self.blocks = new_blocks

    def print_to_file(self, filename="stdout"):
        """
        Print the anchors to a file, optionally filtering them based on the
        accepted pairs.
        """
        fw = must_open(filename, "w")
        for block in self.blocks:
            print("###", file=fw)
            for line in block:
                a, b, score = line
                print("\t".join((a, b, score)), file=fw)
        fw.close()

        logger.debug("Anchors written to `%s`", filename)

    def blast(self, blastfile=None, outfile=None):
        """
        convert anchor file to 12 col blast file
        """
        from ..formats.blast import BlastSlow, BlastLineByConversion

        if not outfile:
            outfile = self.filename + ".blast"

        if blastfile is not None:
            blasts = BlastSlow(blastfile).to_dict()
        else:
            blasts = None

        fw = must_open(outfile, "w", checkexists=True)
        nlines = 0
        for a, b, _ in self.iter_pairs():
            if (a, b) in blasts:
                bline = blasts[(a, b)]
            elif (b, a) in blasts:
                bline = blasts[(b, a)]
            else:
                line = "\t".join((a, b))
                bline = BlastLineByConversion(line, mode="110000000000")

            print(bline, file=fw)
            nlines += 1
        fw.close()

        logger.debug("A total of %d BLAST lines written to `%s`", nlines, outfile)

        return outfile

    @property
    def is_empty(self):
        blocks = self.blocks
        return not blocks or not blocks[0]


def get_best_pair(qs, ss, ts):
    pairs = {}
    for q, s, t in zip(qs, ss, ts):
        t = int(t[:-1]) if t[-1] == "L" else int(t)
        if q not in pairs or pairs[q][1] < t:
            pairs[q] = (s, t)

    # Discard score
    spairs = dict((q, s) for q, (s, t) in pairs.items())
    return spairs


def make_range(q, s, t, i, order, block_pairs, clip=10):
    pairs = get_best_pair(q, s, t)
    score = len(pairs)
    block_pairs[i].update(pairs)

    q = [order[x][0] for x in q]
    q.sort()
    qmin = q[0]
    qmax = q[-1]
    if qmax - qmin >= 2 * clip:
        qmin += clip / 2
        qmax -= clip / 2

    return Range("0", qmin, qmax, score=score, id=i)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
Functions in this script produce figures in the JCVI manuscript.
"""

import sys

import networkx as nx

from ..apps.base import ActionDispatcher, OptionParser, logger
from ..assembly.geneticmap import draw_geneticmap_heatmap
from ..assembly.hic import draw_hic_heatmap
from ..assembly.kmer import draw_ks_histogram
from ..compara.pedigree import Pedigree, calculate_inbreeding
from ..compara.synteny import check_beds
from ..graphics.base import (
    cm,
    load_image,
    normalize_axes,
    panel_labels,
    plt,
    savefig,
    set1,
    setup_theme,
)
from ..graphics.chromosome import draw_chromosomes
from ..graphics.dotplot import dotplot
from ..graphics.karyotype import Karyotype
from ..graphics.landscape import draw_heatmaps, draw_multi_depth, draw_stacks
from ..graphics.synteny import Synteny, draw_gene_legend


def synteny(args):
    """
    %prog synteny grape.peach.anchors seqids layout blocks grape_peach.bed blocks.layout

    Plot synteny composite figure, including:
    A. Synteny dotplot
    B. Karyotype plot
    """
    p = OptionParser(synteny.__doc__)
    p.set_beds()
    opts, args, iopts = p.set_image_options(args, figsize="14x7")
    setup_theme(style="dark")

    if len(args) != 6:
        sys.exit(not p.print_help())

    anchorfile, seqidsfile, layoutfile, datafile, bedfile, blockslayoutfile = args

    fig = plt.figure(1, (iopts.w, iopts.h))
    root = fig.add_axes((0, 0, 1, 1))

    ax1_root = fig.add_axes((0, 0, 0.5, 1))
    ax1_canvas = fig.add_axes((0.05, 0.1, 0.4, 0.8))  # the dot plot
    ax2_root = fig.add_axes((0.5, 0.5, 0.5, 0.5))
    ax3_root = fig.add_axes((0.5, 0, 0.5, 0.5))

    # Panel A
    logger.info("Plotting synteny dotplot")
    qbed, sbed, _, _, is_self = check_beds(anchorfile, p, opts)
    dotplot(
        anchorfile,
        qbed,
        sbed,
        fig,
        ax1_root,
        ax1_canvas,
        is_self=is_self,
        chrlw=0.5,
        sepcolor=set1[3],
    )

    # Panel B
    logger.info("Plotting karyotype plot")
    Karyotype(ax2_root, seqidsfile, layoutfile)

    # Panel C
    logger.info("Plotting synteny blocks")
    Synteny(fig, ax3_root, datafile, bedfile, blockslayoutfile, pad=0.1, vpad=0.03)
    draw_gene_legend(root, 0.69, 0.8, 0.34)

    labels = ((0.02, 0.95, "A"), (0.52, 0.95, "B"), (0.52, 0.45, "C"))
    panel_labels(root, labels)
    normalize_axes(root, ax1_root, ax2_root, ax3_root)

    image_name = "synteny.pdf"
    savefig(image_name, dpi=iopts.dpi, iopts=iopts)


def diversity(args):
    """
    %prog diversity pedigree.ped VAR?_srtd.wgs.regions.bed.gz

    Plot diversity composite figure, including:
    A. Pedigree
    B. Depth distribution across genomes
    """
    p = OptionParser(diversity.__doc__)
    _, args, iopts = p.set_image_options(args, figsize="14x7")

    if len(args) < 2:
        sys.exit(not p.print_help())

    pedfile, bedfiles = args[0], args[1:]

    fig = plt.figure(1, (iopts.w, iopts.h))
    root = fig.add_axes((0, 0, 1, 1))

    ax1_root = fig.add_axes((0, 0, 0.25, 1))
    ax2_root = fig.add_axes((0.25, 0, 0.75, 1))

    # Panel A
    logger.info("Plotting pedigree")
    ped = Pedigree(pedfile)
    pngfile = f"{pedfile}.png"
    inb = calculate_inbreeding(ped, ploidy=4, N=10000)

    G = ped.to_graph(inb, title="Pedigree of Variety1")
    A = nx.nx_agraph.to_agraph(G)
    dpi = 300
    A.draw(pngfile, prog="dot", args=f"-Gdpi={dpi}")
    logger.info("Pedigree graph written to `%s`", pngfile)

    # Show the image as is
    ax1_root.imshow(load_image(pngfile))
    ax1_root.set_axis_off()

    # Panel B
    logger.info("Plotting depth distribution across genomes")
    npanels = len(bedfiles)
    yinterval = 1.0 / npanels
    ypos = 1 - yinterval
    panel_roots, panel_axes = [], []
    for _ in range(npanels):
        panel_root = fig.add_axes((0.25, ypos, 0.75, yinterval))
        panel_ax = fig.add_axes(
            (0.25 + 0.1 * 0.75, ypos + 0.2 * yinterval, 0.8 * 0.75, 0.65 * yinterval)
        )
        panel_roots.append(panel_root)
        panel_axes.append(panel_ax)
        ypos -= yinterval

    draw_multi_depth(
        ax2_root,
        panel_roots,
        panel_axes,
        bedfiles,
        chrinfo_file="chrinfo.txt",
        titleinfo_file="titleinfo.txt",
        maxdepth=100,
        logscale=False,
    )

    labels = (
        (0.02, 0.95, "A"),
        (0.25 + 0.25 * 0.1, 0.95, "B"),
    )
    panel_labels(root, labels)
    normalize_axes(root, ax2_root)

    image_name = "diversity.pdf"
    savefig(image_name, dpi=iopts.dpi, iopts=iopts)


def landscape(args):
    """
    %prog landscape features.bed athaliana.sizes TAIR10_chr_all.fas Chr2

    Plot landscape composite figure, including:
    A. Example genomic features painted on Arabidopsis genome
    B. Landscape of genomic features across the genome
    """
    p = OptionParser(landscape.__doc__)
    _, args, iopts = p.set_image_options(args, figsize="12x8")

    if len(args) != 4:
        sys.exit(not p.print_help())

    bedfile, sizesfile, fastafile, ch = args

    fig = plt.figure(1, (iopts.w, iopts.h))
    root = fig.add_axes((0, 0, 1, 1))
    aspect_ratio = iopts.w / iopts.h
    ax1_root = fig.add_axes((0, 1 / 4, 0.4, 0.5 * aspect_ratio))
    ax2_root_extent = (0.4, 0.5, 0.6, 0.47)
    ax2_root = fig.add_axes(ax2_root_extent)
    ax3_root_extent = (0.41, 0, 0.6, 0.47)
    ax3_root = fig.add_axes(ax3_root_extent)

    # Panel A
    logger.info("Plotting example genomic features painted on Arabidopsis genome")
    draw_chromosomes(
        ax1_root,
        bedfile,
        sizesfile,
        iopts=iopts,
        mergedist=0,
        winsize=50000,
        gauge=True,
        legend=True,
        empty=False,
        title="*Arabidopsis* genome features",
    )

    # Panel B
    logger.info("Plotting landscape of genomic features across the genome")
    stacks = ["Repeats", "Exons"]
    heatmaps = ["Copia", "Gypsy", "Helitron", "hAT", "Exons"]
    window = 250000
    shift = 50000
    draw_stacks(
        fig,
        ax2_root,
        ax2_root_extent,
        stacks,
        fastafile,
        window,
        shift,
        top=5,
    )

    # Panel C
    draw_heatmaps(
        fig,
        ax3_root,
        ax3_root_extent,
        fastafile,
        "Chr2",
        stacks,
        heatmaps,
        window,
        shift,
        cmap=cm.viridis,
    )

    ax2_root.set_axis_off()
    ax3_root.set_axis_off()

    labels = ((0.02, 0.95, "A"), (0.42, 0.95, "B"), (0.42, 0.48, "C"))
    panel_labels(root, labels)
    normalize_axes(root, ax1_root)

    image_name = "landscape.pdf"
    savefig(image_name, dpi=iopts.dpi, iopts=iopts)


def genomebuild(args):
    """
    %prog genomebuild reads.histo geneticmap.matrix hic.resolution_500000.npy hic.resolution_500000.json

    Plot genome build composite figure, including:
    A. Read kmer histogram
    B. Genetic map concordance
    C. Hi-C contact map concordance
    """
    p = OptionParser(genomebuild.__doc__)
    _, args, iopts = p.set_image_options(args, figsize="21x7")

    if len(args) != 4:
        sys.exit(not p.print_help())

    reads_histo, mstmap, hic_matrix, hic_json = args

    fig = plt.figure(1, (iopts.w, iopts.h))
    root = fig.add_axes((0, 0, 1, 1))
    ax1_root = fig.add_axes((0, 0, 1 / 3, 1))
    ax2_root = fig.add_axes((1 / 3, 0, 1 / 3, 1))
    ax3_root = fig.add_axes((2 / 3, 0, 1 / 3, 1))
    ax1 = fig.add_axes((1 / 3 * 0.1, 0.1, 1 / 3 * 0.8, 0.8))
    ax2 = fig.add_axes((1 / 3 * 1.1, 0.1, 1 / 3 * 0.8, 0.8))
    ax3 = fig.add_axes((1 / 3 * 2.1, 0.1, 1 / 3 * 0.8, 0.8))

    # Panel A
    logger.info("Plotting read kmer histogram")
    _ = draw_ks_histogram(
        ax1,
        reads_histo,
        method="nbinom",
        coverage=0,
        vmin=2,
        vmax=200,
        species="*S. species* ‘Variety 1’",
        K=21,
        maxiter=100,
        peaks=False,
    )

    # Panel B
    logger.info("Plotting genetic map concordance")
    draw_geneticmap_heatmap(ax2_root, ax2, mstmap, 1000)

    # Panel C
    logger.info("Plotting Hi-C contact map concordance")
    draw_hic_heatmap(
        ax3_root,
        ax3,
        hic_matrix,
        hic_json,
        contig=None,
        groups_file="groups",
        title="*S. species* Hi-C contact map",
        vmin=1,
        vmax=6,
        plot_breaks=True,
    )

    labels = (
        (1 / 3 * 0.1, 0.95, "A"),
        (1 / 3 * 1.1, 0.95, "B"),
        (1 / 3 * 2.1, 0.95, "C"),
    )
    panel_labels(root, labels)
    normalize_axes(root, ax1_root, ax2_root, ax3_root)

    image_name = "genomebuild.pdf"
    savefig(image_name, dpi=iopts.dpi, iopts=iopts)


def main():

    actions = (
        ("synteny", "Plot synteny composite figure"),
        ("diversity", "Plot diversity composite figure"),
        ("genomebuild", "Plot genome build composite figure"),
        ("landscape", "Plot landscape composite figure"),
    )
    p = ActionDispatcher(actions)
    p.dispatch(globals())


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
%prog seqids layout

Illustrate macrosynteny between tracks which represent individual genomes.

seqids contain the chromosomes to plot. Each line correspond to a track.
layout provides configuration for placement of tracks and mapping file between tracks.

Layout file example - first section specify how to draw each track. Then the "edges"
section specify which connections to draw.

# y, xstart, xend, rotation, color, label, va, bed, label_va
.6, .1, .4, 0, m, Grape, top, grape.bed, center
.4, .3, .6, 60, k, Athaliana, top, athaliana.bed, center
# edges
e, 0, 1, athaliana.grape.4x1.simple
"""


import sys

from typing import List, Optional

from ..apps.base import OptionParser, logger
from ..compara.synteny import SimpleFile
from ..formats.bed import Bed

from .base import (
    AbstractLayout,
    markup,
    mpl,
    normalize_axes,
    plt,
    savefig,
    update_figname,
)
from .chromosome import Chromosome, HorizontalChromosome
from .glyph import TextCircle
from .synteny import Shade, ymid_offset


class LayoutLine(object):
    def __init__(self, row, delimiter=",", generank=True):
        args = row.rstrip().split(delimiter)
        args = [x.strip() for x in args]

        self.empty = False
        if len(args) < 8:
            self.empty = True
            return
        self.y = float(args[0])
        self.xstart = float(args[1])
        self.xend = float(args[2])
        self.rotation = int(args[3])
        self.color = args[4]
        self.label = args[5]
        self.va = args[6]
        self.bed = Bed(args[7])
        if len(args) == 9:
            self.label_va = args[8]
        else:
            self.label_va = "center"
        self.order = self.bed.order
        self.order_in_chr = self.bed.order_in_chr if generank else self.bed.bp_in_chr


class Layout(AbstractLayout):
    def __init__(
        self, filename, delimiter=",", generank=False, seed: Optional[int] = None
    ):
        super().__init__(filename)
        fp = open(filename)
        self.edges = []
        for row in fp:
            if row[0] == "#":
                continue
            if row[0] == "e":
                args = row.rstrip().split(delimiter)
                args = [x.strip() for x in args]
                i, j, fn = args[1:4]
                if len(args) == 5 and args[4]:
                    samearc = args[4]
                else:
                    samearc = None
                i, j = int(i), int(j)
                assert args[0] == "e"
                blocks = self.parse_blocks(fn, i)
                self.edges.append((i, j, blocks, samearc))
            else:
                self.append(LayoutLine(row, delimiter=delimiter, generank=generank))

        self.assign_colors(seed=seed)

    def parse_blocks(self, simplefile, i):
        order = self[i].order
        return SimpleFile(simplefile, order=order).blocks


MaxSeqids = 16  # above which no labels are written


def make_circle_name(sid, rev):
    """Extract a succinct label based on sid.

    If there are numbers to be extracted, returns the first group of number.
    Otherwise, the first letter is returned.

    If sid is in rev, then '-' gets appended to the label.

    Args:
        sid (str): seqid
        rev (set[str]): Set of seqids that are reversed

    Returns:
        str: Single letter label for the sid
    """
    import re

    in_reverse = sid in rev
    sid = sid.rsplit("_", 1)[-1]
    sid = sid.replace("chr", "").replace("Chr", "")
    si = re.findall(r"\d+", sid)
    if si:
        si = str(int(si[0]))
    else:
        si = sid[0].upper()
    if in_reverse:
        si += "-"
    return si


class Track(object):
    def __init__(
        self,
        ax,
        t,
        gap=0.01,
        height=0.01,
        lw=1,
        draw=True,
        chrstyle="auto",
    ):
        self.empty = t.empty
        if t.empty:
            return

        # Copy the data from LayoutLine
        self.y = t.y
        self.sizes = sizes = t.sizes
        self.label = t.label
        self.rotation = t.rotation
        self.va = t.va
        self.label_va = t.label_va
        self.color = t.color if t.color != "None" else None
        self.seqids = t.seqids
        self.bed = t.bed
        self.order = t.order
        self.order_in_chr = t.order_in_chr
        self.rev = t.rev
        self.ax = ax
        self.height = height

        self.xstart = xstart = t.xstart
        self.xend = t.xend

        # Rotation transform
        self.x = x = (self.xstart + self.xend) / 2
        y = self.y
        self.tr = (
            mpl.transforms.Affine2D().rotate_deg_around(x, y, self.rotation)
            + ax.transAxes
        )
        self.inv = ax.transAxes.inverted()

        nseqids = len(self.seqids)
        if nseqids > MaxSeqids:
            gap = min(gap, gap * MaxSeqids / nseqids + 0.001)
        self.gap = gap

        rpad = 1 - t.xend
        span = 1 - xstart - rpad - gap * (len(sizes) - 1)
        self.total = total = sum(sizes.values())
        ratio = span / total

        self.ratio = ratio
        self.update_offsets()
        self.lw = lw

        if draw:
            self.draw(chrstyle=chrstyle)

    def __str__(self):
        return self.label

    def draw(
        self,
        chrstyle="auto",
        keep_chrlabels=False,
        plot_label=True,
        plot_circles=True,
        pad=0.03,
        vpad=0.09,
    ):
        if self.empty:
            return

        y = self.y
        color = self.color
        ax = self.ax
        xstart = self.xstart
        gap = self.gap
        va = self.va
        nseqids = len(self.seqids)
        tr = self.tr

        for i, sid in enumerate(self.seqids):
            size = self.sizes[sid]
            rsize = self.ratio * size
            xend = xstart + rsize
            hc = HorizontalChromosome(
                ax,
                xstart,
                xend,
                y,
                height=self.height,
                lw=self.lw,
                fc=color,
                style=chrstyle,
            )
            hc.set_transform(tr)
            si = sid if keep_chrlabels else make_circle_name(sid, self.rev)
            xx = (xstart + xend) / 2
            xstart = xend + gap

            step = 2 if nseqids <= 40 else 10
            if nseqids >= 2 * MaxSeqids and (i + 1) % step != 0:
                continue
            if nseqids < 5:
                continue

            hpad = -pad if va == "bottom" else pad
            if plot_circles:
                TextCircle(
                    ax,
                    xx,
                    y + hpad,
                    si,
                    fc="w",
                    color=color,
                    size=10,
                    transform=tr,
                )

        label = markup(self.label)
        c = color if color != "gainsboro" else "k"
        if plot_label:
            if self.label_va == "top":
                x, y = self.x, self.y + vpad
            elif self.label_va == "bottom":
                x, y = self.x, self.y - vpad
            else:  # "center"
                x, y = self.xstart - vpad / 2, self.y
            ax.text(x, y, label, ha="center", va="center", color=c, transform=tr)

    def update_offsets(self):
        self.offsets = {}
        xs = self.xstart
        gap = self.gap
        for sid in self.seqids:
            size = self.sizes[sid]
            self.offsets[sid] = xs
            xs += self.ratio * size + gap

    def get_coords(self, gene):
        order_in_chr = self.order_in_chr
        seqid, i, _ = order_in_chr[gene]
        if seqid not in self.offsets:
            return [None, None]

        x = self.offsets[seqid]
        if seqid in self.rev:
            x += self.ratio * (self.sizes[seqid] - i - 1)
        else:
            x += self.ratio * i
        y = self.y
        x, y = self.tr.transform((x, y))
        x, y = self.inv.transform((x, y))

        return [x, y]


class ShadeManager(object):
    def __init__(self, ax, tracks, layout, heightpad=0, style="curve"):
        self.style = style
        for i, j, blocks, samearc in layout.edges:
            # if same track (duplication shades), shall we draw above or below?
            self.draw_blocks(
                ax, blocks, tracks[i], tracks[j], samearc=samearc, heightpad=heightpad
            )

    def draw_blocks(
        self, ax, blocks, atrack, btrack, samearc: Optional[str], heightpad=0
    ):
        for a, b, c, d, _, _, highlight in blocks:
            p = atrack.get_coords(a), atrack.get_coords(b)
            q = btrack.get_coords(c), btrack.get_coords(d)
            if p[0] is None or q[0] is None:
                continue

            ymid_pad = ymid_offset(samearc)
            if heightpad:
                if atrack.y < btrack.y:
                    p[0][1] = p[1][1] = atrack.y + heightpad
                    q[0][1] = q[1][1] = btrack.y - heightpad
                else:
                    p[0][1] = p[1][1] = atrack.y - heightpad
                    q[0][1] = q[1][1] = btrack.y + heightpad

            zorder = 2 if highlight else 1
            lw = 1 if highlight else 0
            Shade(
                ax,
                p,
                q,
                ymid_pad,
                highlight=highlight,
                alpha=1,
                fc="gainsboro",
                ec="gainsboro",
                lw=lw,
                zorder=zorder,
                style=self.style,
            )


class Karyotype(object):
    def __init__(
        self,
        root,
        seqidsfile,
        layoutfile,
        gap=0.01,
        height=0.01,
        lw=1,
        generank=True,
        sizes=None,
        heightpad=0,
        keep_chrlabels=False,
        plot_label=True,
        plot_circles=True,
        shadestyle="curve",
        chrstyle="auto",
        seed: Optional[int] = None,
    ):
        layout = Layout(layoutfile, generank=generank, seed=seed)

        fp = open(seqidsfile)
        # Strip the reverse orientation tag for e.g. chr3-
        di = lambda x: x[:-1] if x[-1] == "-" else x
        # Comments can cause layout and seqids to be out of sync
        # https://github.com/tanghaibao/jcvi/issues/676
        for i, row in enumerate(_ for _ in fp if not _.startswith("#") and _.strip()):
            logger.info("Processing `%s` (track %d)", row.strip(), i)
            t = layout[i]
            # There can be comments in seqids file:
            # https://github.com/tanghaibao/jcvi/issues/335
            seqids = row.split("#", 1)[0].rstrip().split(",")
            t.rev = set(x[:-1] for x in seqids if x[-1] == "-")
            seqids = [di(x) for x in seqids]
            if t.empty:
                continue

            bed = t.bed
            self.generank = generank
            if generank:
                sz = dict((x, len(list(bed.sub_bed(x)))) for x in seqids)
            else:
                sz = sizes or dict(
                    (x, max(z.end for z in list(bed.sub_bed(x)))) for x in seqids
                )
                assert sz is not None, "sizes not available and cannot be inferred"
            t.seqids = seqids
            # validate if all seqids are non-empty
            for k, v in sz.items():
                if v == 0:
                    logger.error("Size of `%s` is empty. Please check", k)
            t.sizes = sz

        tracks = []
        for lo in layout:
            if lo.empty:
                continue
            tr = Track(root, lo, gap=gap, height=height, lw=lw, draw=False)
            tracks.append(tr)

        ShadeManager(root, tracks, layout, heightpad=heightpad, style=shadestyle)

        for tr in tracks:
            tr.draw(
                chrstyle=chrstyle,
                keep_chrlabels=keep_chrlabels,
                plot_label=plot_label,
                plot_circles=plot_circles,
            )

        self.tracks = tracks
        self.layout = layout


def main(args: List[str]):
    p = OptionParser(__doc__)
    p.add_argument(
        "--basepair",
        default=False,
        action="store_true",
        help="Use base pair position instead of gene rank",
    )
    p.add_argument(
        "--keep-chrlabels",
        default=False,
        action="store_true",
        help="Keep chromosome labels",
    )
    p.add_argument(
        "--nocircles",
        default=False,
        action="store_true",
        help="Do not plot chromosome circles",
    )
    p.add_argument(
        "--shadestyle",
        default="curve",
        choices=Shade.Styles,
        help="Style of syntenic wedges",
    )
    p.add_argument(
        "--chrstyle",
        default="auto",
        choices=Chromosome.Styles,
        help="Style of chromosome labels",
    )
    p.set_outfile("karyotype.pdf")
    opts, args, iopts = p.set_image_options(args, figsize="8x7")

    if len(args) != 2:
        sys.exit(not p.print_help())

    seqidsfile, layoutfile = args

    fig = plt.figure(1, (iopts.w, iopts.h))
    root = fig.add_axes((0, 0, 1, 1))

    Karyotype(
        root,
        seqidsfile,
        layoutfile,
        keep_chrlabels=opts.keep_chrlabels,
        plot_circles=(not opts.nocircles),
        shadestyle=opts.shadestyle,
        chrstyle=opts.chrstyle,
        generank=(not opts.basepair),
        seed=iopts.seed,
    )
    normalize_axes(root)

    image_name = update_figname(opts.outfile, iopts.format)
    savefig(image_name, dpi=iopts.dpi, iopts=iopts)

    return image_name


if __name__ == "__main__":
    main(sys.argv[1:])



from datetime import datetime
from importlib.metadata import version, PackageNotFoundError


__author__ = (
    "Haibao Tang",
    "Vivek Krishnakumar",
    "Xingtan Zhang",
    "Won Cheol Yim",
)
__copyright__ = f"Copyright (c) 2010-{datetime.now().year}, Haibao Tang"
__email__ = "tanghaibao@gmail.com"
__license__ = "BSD"
__status__ = "Development"


try:
    VERSION = version(__name__)
except PackageNotFoundError:  # pragma: no cover
    try:
        from .version import version as VERSION  # noqa
    except ImportError as exc:  # pragma: no cover
        raise ImportError(
            "Failed to find (autogenerated) version.py. "
            "This might be because you are installing from GitHub's tarballs, "
            "use the PyPI ones."
        ) from exc
__version__ = VERSION



#!/usr/bin/env python

"""Package setup"""

from Cython.Build import build_ext
from setuptools import setup, Extension

import numpy as np

ext_modules = [
    Extension(
        "jcvi.assembly.chic",
        ["jcvi/assembly/chic.pyx"],
        include_dirs=[np.get_include()],
        extra_compile_args=["-O3"],
    ),
    Extension(
        "jcvi.formats.cblast", ["jcvi/formats/cblast.pyx"], extra_compile_args=["-O3"]
    ),
]

if __name__ == "__main__":
    setup(
        cmdclass={"build_ext": build_ext},
        ext_modules=ext_modules,
    )
